package com.dtolabs.rundeck.core.authorization;

import com.dtolabs.rundeck.core.Constants;
import com.dtolabs.rundeck.core.authentication.Group;
import com.dtolabs.rundeck.core.authentication.Username;
import com.dtolabs.rundeck.core.authorization.providers.EnvironmentalContext;
import com.dtolabs.rundeck.core.authorization.providers.PoliciesParseException;
import com.dtolabs.rundeck.core.common.Framework;

import javax.security.auth.Subject;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.util.*;

/**
 * Created with IntelliJ IDEA.
 * User: greg
 * Date: 3/13/13
 * Time: 6:27 PM
 * To change this template use File | Settings | File Templates.
 */
public class CheckAuth {
    public static void main(String... args) throws IOException, PoliciesParseException {
        System.out.println("Press enter to continue:");
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(System.in));
        while (null != bufferedReader.readLine()) {
            System.out.println("Doing test...");
            doTest(args);
        }
    }

    public static void doTest(String... args) throws IOException, PoliciesParseException {
        int i = 0;
        String rdeckbase = args[i++];
        String projectName = args[i++];

        String user = args[i++];
        String[] roles = args[i++].split(",\\s*");

        Framework framework = buildFramework(rdeckbase);
        SingleUserAclsAuthorization singleUserAclsAuthorization = new SingleUserAclsAuthorization(framework,
                new File(Constants.getFrameworkConfigDir(rdeckbase)), rdeckbase, roles);

        String[] actions = args[i++].split(",\\s*");
//        String action = args[i++];
        HashSet<String> actionSet = new HashSet<String>(Arrays.asList(actions));

        Map<String, String> resmap = new HashMap<String, String>();
        String param = null;
        for (; i < args.length; i++) {
            String[] z = args[i].split("=", 2);
            if (z.length > 1) {
                resmap.put(z[0], z[1]);
                if (z[1].contains("#")) {
                    param = z[0];
                }
            }
        }
        Set<Map<String, String>> resmapset = new HashSet<Map<String, String>>();
        int max = 200;
        if (param != null) {
            //parameterized resources for multiple checks
            String value = resmap.get(param).split("#", 2)[1];
            max = Integer.parseInt(value);
            for (int x = 1; x <= max; x++) {
                Map<String, String> resmap2 = new HashMap<String, String>(resmap);
                String newval = resmap2.get(param).replaceAll("#\\d+", Integer.toString(x));
                resmap2.put(param, newval);
                resmapset.add(resmap2);
            }
        } else {
            resmapset.add(resmap);
        }
        Set<Attribute> attributes;
        if (projectName.equalsIgnoreCase("--application")) {
            attributes = Collections.singleton(new Attribute(URI.create(EnvironmentalContext.URI_BASE + "application"), "rundeck"));
        } else {
            attributes = Collections.singleton(new Attribute(URI.create(EnvironmentalContext.URI_BASE + "project"), projectName));
        }
        Subject subject = createSubject(user, roles);
        long warmup = timeCheck(singleUserAclsAuthorization, actionSet, resmapset, attributes, subject);
        System.out.println("warmup: " + warmup);

        long total = 0;
        int testcount = 20;
        for (int x = 0; x < testcount; x++) {
            total += timeCheck(singleUserAclsAuthorization, actionSet, resmapset, attributes, subject);
        }
        System.out.println("Count(" + testcount + ") resMapsize(" + resmapset.size() + ") avg: " + (total / testcount));
//        System.out.println("Cache: " + singleUserAclsAuthorization.cacheReport());
    }

    private static long timeCheck(SingleUserAclsAuthorization singleUserAclsAuthorization, HashSet<String> actionSet, Set<Map<String, String>> resmapset, Set<Attribute> attributes, Subject subject) {
        Set<Decision> evaluate = null;
        long time0 = System.currentTimeMillis();

        long time = System.currentTimeMillis();
        evaluate = singleUserAclsAuthorization.evaluate(resmapset, subject, actionSet, attributes);
        long now = System.currentTimeMillis();
        long now0 = System.currentTimeMillis();
        long result = now0 - time0;
        return result;
    }


    private static Subject createSubject(String user, String[] rolelist) {
        Subject subject = new Subject();
        subject.getPrincipals().add(new Username(user));
        for (String s : rolelist) {
            subject.getPrincipals().add(new Group(s));
        }
        return subject;
    }

    private static Framework buildFramework(String rdeckbase) {
        return Framework.getInstance(rdeckbase);
    }

    public static class SingleUserAclsAuthorization extends BaseAclsAuthorization {

        private String username;
        private String[] roles;

        public SingleUserAclsAuthorization(final Framework framework, final File basedir, String username, String[] roles)
                throws IOException, PoliciesParseException {
            super(framework, basedir);
            this.username = username;
            this.roles = roles;
        }

        public String[] determineUserRoles(String user) {
            if (username.equals(user)) {
                return roles;
            } else {
                return new String[0];

            }
        }

//        public String cacheReport() {
//            return this.authorization.cacheReport().toString();
//        }
    }
}
